@@ -4,9 +4,11 @@ Database related utilities
 """
 import logging
 from contextlib import contextmanager
+from pathlib import Path
 from typing import ContextManager
 
 from sqlalchemy import create_engine
+from sqlalchemy.exc import SQLAlchemyError
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.orm import sessionmaker
 from sqlalchemy.orm.session import Session
@@ -16,7 +18,7 @@ logger = logging.getLogger(__name__)
 Base = declarative_base()
 
 
-def db_init(db_file: str) -> sessionmaker:
+def db_init(db_file: Path) -> sessionmaker:
     """
     Initializes the database and returns a session factory
     """
@@ -41,7 +43,7 @@ def db_session(session_factory: sessionmaker) -> ContextManager[Session]:
         if session.new or session.dirty or session.deleted:
             session.commit()
 
-    except Exception as exc:
+    except SQLAlchemyError as exc:
         logger.exception(exc)
         session.rollback()
 
