@@ -4,7 +4,6 @@ from sqlalchemy import Column, DateTime, ForeignKey, Integer
 from sqlalchemy.orm import relationship
 
 from libratom.lib.database import Base
-from libratom.models.file_report import FileReport
 
 
 class Message(Base):
@@ -12,7 +11,9 @@ class Message(Base):
 
     id = Column(Integer, primary_key=True)
     pff_identifier = Column(Integer)
+    date = Column(DateTime)
     processing_start_time = Column(DateTime)
     processing_end_time = Column(DateTime)
-    file_report_id = Column(Integer, ForeignKey(FileReport.id))
+    file_report_id = Column(Integer, ForeignKey("file_report.id"))
     entities = relationship("Entity", backref="message")
+    attachments = relationship("Attachment", backref="message")
